

=Our first chapter

Let's say I want to \X<index> a \X<word>.

It is a very \X<important> one everyone should know about.

\INDEXRELATIONS{readdepth=full reldepth=startpage threshold="50%" format=bullets type=linked}

==... with deeper explanations

You know, an index collects keywords of the \X<text>.


=Our second chapter

There are more \X<word>s out of course, but they are not
of interest in this \X<text>.

\INDEXRELATIONS{threshold="100%" format=enumerated type=plain}




=Conclusion

Importance is (con)\X<text> dependent.

\INDEXRELATIONS{format=numbers}


=Index

\INDEX




