

=A two stream doc

This document compares two imaginary objects.

~The ignored docstream

This part shall not be made part of the document.

~The first object

Manufacturer, price, and more common data of the 1st item.

~The 2nd object

Manufacturer, price, and more common data of the 2nd item.


==Advantages

What they are good in.

~The first object

Advantages of this 1st item.

~The ignored docstream

This part shall not be made part of the document.


~The 2nd object

Advantages of this 2nd item.



==Suggestions

Talks about things to be improved.

~The first object

1st item can be improved this way.

~The 2nd object

2nd item can be improved this way.

~The ignored docstream

? 0

This part shall not be made part of the document.
Therefore the condition above should be ignored.



==Conclusion

What the editors think and suggest.

~The first object
A short summary about item 1.

~The 2nd object
A short summary about item 2.


